<?php

namespace Drupal\something_went_wrong\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\something_went_wrong\SomethingWentWrongServiceInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class Something went wrong D9 Exception Subscriber.
 *
 * @package Drupal\something_went_wrong\EventSubscriber
 */
class SomethingWentWrongExceptionSubscriber implements EventSubscriberInterface {

  /**
   * The module config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $moduleConfig;

  /**
   * The Something went wrong service.
   *
   * @var \Drupal\something_went_wrong\SomethingWentWrongServiceInterface
   */
  protected SomethingWentWrongServiceInterface $somethingWentWrongService;

  /**
   * SomethingWentWrongD9ExceptionSubscriber constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\something_went_wrong\SomethingWentWrongServiceInterface $something_went_wrong_service
   *   The Something went wrong service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    SomethingWentWrongServiceInterface $something_went_wrong_service,
  ) {
    $this->moduleConfig = $config_factory->get('something_went_wrong.settings');
    $this->somethingWentWrongService = $something_went_wrong_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::EXCEPTION][] = ['onException', 60];
    return $events;
  }

  /**
   * Exceptions handler.
   *
   * @param \Symfony\Component\HttpKernel\Event\ExceptionEvent $event
   *   The event to process.
   */
  public function onException(ExceptionEvent $event): void {
    $exception = $event->getThrowable();
    $class_name = get_class($exception);
    $ignore_list = explode(',', $this->moduleConfig->get('ignore_list'));

    $send = TRUE;

    foreach ($ignore_list as $item) {
      if ($item && str_contains($class_name, trim($item))) {
        $send = FALSE;
        break;
      }
    }

    if ($send) {
      if ($this->moduleConfig->get('slack')) {
        $message = $this->somethingWentWrongService->buildMessage($exception);
        $this->somethingWentWrongService->sendMessageToSlack($message);
      }

      if ($this->moduleConfig->get('mail')) {
        $message = $this->somethingWentWrongService->buildMessage($exception, TRUE);
        $this->somethingWentWrongService->sendMessageToMail($message);
      }
    }
  }

}
